// ----------------------------------------------------------------------------
// Next.java
// The "next" command.
// $Id: Next.java,v 1.4 2006/03/29 15:49:47 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;

import fmrisc.Proving.*;

/**
 * 
 * The "next" command: goto next open proof state.
 *
 */
public final class Next extends ProofCommandBase
{
  public final static String Name = "next";
  
  /***************************************************************************
   * Create a "next" command.
   **************************************************************************/
  public Next()
  {
    super(Name);
  }
  
  /****************************************************************************
   * Process command "next": goto next open proof state.
   * @param state the proof state to which the command is applied
   * @return null.
   ***************************************************************************/
  protected ProofState[] processCore(ProofState state)
  {
    Proof proof = state.getProof();
    proof.nextOpenState();
    return null;
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
  }
}