// ----------------------------------------------------------------------------
// ArrayType.java
// Handling of array types.
// $Id: ArrayType.java,v 1.12 2006/03/14 19:13:03 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * Handling of array types.
 */
public final class ArrayType extends TypeBase
{
  private Type index;  // index type
  private Type base;   // base type

  /****************************************************************************
   * construct array type with denoted index type and base type.
   * @param index the index type of the array
   * @param base the base type of the array
   ***************************************************************************/
  public ArrayType(Type index, Type base)
  {
    this.index = index;
    this.base = base;
  }
  
  /****************************************************************************
   * returns index type
   * @return the index type
   ***************************************************************************/
  public Type getIndex()
  {
    return index;
  }
  
  /****************************************************************************
   * returns base type
   * @return the base type
   ***************************************************************************/
  public Type getBase()
  {
    return base;
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Type instantiateType()
  {
    return Construct.arrayType(getIndex().instantiateType(),
        getBase().instantiateType());
  }
  
  /***************************************************************************
   * Prints text representation of type on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print("ARRAY ");
    index.print(out);
    out.print(" OF ");
    base.print(out);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null).
   **************************************************************************/
  public AST accept(ASTVisitor visitor)
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    AST index0 = index.accept(visitor);
    AST base0 = base.accept(visitor);
    if (!visitor.isCloned()) return null;
    return Construct.arrayType((Type)index0, (Type)base0);
  }
}
