// ----------------------------------------------------------------------------
// AtomicType.java
// Handling of atomic types.
// $Id: AtomicType.java,v 1.11 2006/02/23 19:51:06 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * Handling of atomic types.
 */
public final class AtomicType extends TypeBase
{
  // the name of the atomic type
  private String name;
  
  /****************************************************************************
   * construct atomic type with denoted name.
   * @param name the name of the type
   ***************************************************************************/
  public AtomicType(String name)
  {
    this.name = name;
  }
  
  /****************************************************************************
   * returns name of identifier
   * @return the identifier name
   ***************************************************************************/
  public String getName()
  {
    return name;
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Type instantiateType()
  {
    return this;
  }
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(name);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null).
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    if (!visitor.isCloned()) return null;
    return this;
  }
}
